<?php


/**
 * Table schema for saved posts
 */
define('ELOQUA_SCHEMA_SAVED_POSTS', 'eloqua_saved_posts');


/**
 * Table schemo for eloqua enabled webform data
 */
define('ELOQUA_SCHEMA_WEBFORM', 'eloqua_webform');


define('ELOQUA_POST_FIELD_STATUS', 'status');
define('ELOQUA_POST_FIELD_DATA', 'data');
define('ELOQUA_POST_FIELD_FORM_ID', 'form_id');
define('ELOQUA_POST_FIELD_POST_TIME', 'post_time');
define('ELOQUA_POST_FIELD_POST_ID', 'post_id');

define('ELOQUA_WEBFORM_FIELD_NODE_ID', 'nid');
define('ELOQUA_WEBFORM_FIELD_ACTIVE', 'is_active');
define('ELOQUA_WEBFORM_FIELD_FORM_NAME', 'form_name');
define('ELOQUA_WEBFORM_FIELD_REQUEST_NAME', 'request_name');
define('ELOQUA_WEBFORM_FIELD_DATA', 'data');

define('ELOQUA_SETTINGS_FIELD_KEY', 'e_key');
define('ELOQUA_SETTINGS_FIELD_DATA', 'data');

define('ELOQUA_SETTINGS_BATCH_SIZE', 'batch_size');
define('ELOQUA_SETTINGS_BATCH_SIZE_DEFAULT', 50);

define('ELOQUA_STATUS_NEW', 'new');
define('ELOQUA_STATUS_FAILED', 'failed');
define('ELOQUA_STATUS_UPLOADED', 'uploaded');


/**
 * Drupal variable constant that defines the location of the js files
 */
define('ELOQUA_VARIABLE_SCRIPTS_DIRECTORY', 'eloqua_scripts_directory');


/**
 * Default script location value
 */
define('ELOQUA_VARIABLE_SCRIPTS_DIRECTORY_DEFAULT', 'sites/all/libraries/elqNow');

/**
 * Last post time to Eloqua
 */
define('ELOQUA_VARIABLE_LAST_POST_TIME', 'eloqua_last_post_time');

/**
 * Default value for last post time.
 */
define('ELOQUA_VARIABLE_LAST_POST_TIME_DEFAULT', '0');

/**
 * Eloqua Posting Interval
 */
define('ELOQUA_VARIABLE_POST_INTERVAL', 'eloqua_post_interval');

/**
 * Eloqua Site ID
 */
define('ELOQUA_VARIABLE_SITE_ID', 'eloqua_site_id');

/**
 * Default Elqoqua Site ID
 */
define('ELOQUA_VARIABLE_SITE_ID_DEFAULT', 0);


/**
 * Default Eloqua Posting Interval
 */
define('ELOQUA_VARIABLE_POST_INTERVAL_DEFAULT', 86400);


define('ELOQUA_QUERY_OPTION_UNSERIALIZE_DATA', 'unserialize_data');

/**
 * Watchdog Category Key
 */
define('ELOQUA_WATCHDOG_CATEGORY', 'eloqua');


////////////////////////////////////////////////////////////////////////////////
//  VARIABLE ACCESS FUNCTIONS

/**
 * Returns the last post time to the Eloqua System
 * @return int
 *  Timestamp
 */
function eloqua_get_last_post_time() {
  return variable_get(ELOQUA_VARIABLE_LAST_POST_TIME, ELOQUA_VARIABLE_LAST_POST_TIME_DEFAULT);
}


/**
 * Sets the last post time
 * @param $value int
 *  Timestamp
 * @return boolean
 *  Successful update
 */
function eloqua_set_last_post_time($value) {
  if (!is_numeric($value)) {
    $message = t('Invalid value set for post time: !value');
    $variables = array(
      '!value' => '<pre>' . var_export ($value, TRUE) . '</pre>',
    );
    watchdog(ELOQUA_WATCHDOG_CATEGORY, $message, $variables, WATCHDOG_NOTICE);
    return FALSE;
  }
  variable_set(ELOQUA_VARIABLE_LAST_POST_TIME, $value);
  return TRUE;
}


/**
 * Returns the Eloqua Site ID
 * @return int
 */
function eloqua_get_site_id() {
  $site_id = variable_get(ELOQUA_VARIABLE_SITE_ID, ELOQUA_VARIABLE_SITE_ID_DEFAULT);
  return $site_id;
}


/**
 * Gets the update interval
 * @return int
 *   Seconds
 */
function eloqua_get_update_time_interval () {
  return variable_get(ELOQUA_VARIABLE_POST_INTERVAL, ELOQUA_VARIABLE_POST_INTERVAL_DEFAULT);
}


/**
 * Sets the update interval
 * @param $seconds int
 *  Number of seconds to elapse between cron pushes
 * @return boolean
 *  Successful Update
 */
function eloqua_set_update_time_interval ($seconds) {
  if (!is_numeric($seconds)
      || $seconds < 0) {
    $message = t('!method: Invalid value set for post time: !value');
    $variables = array(
      '!method' => __METHOD__,
      '!value' => '<pre>' . var_export ($seconds, TRUE) . '</pre>',
    );
    watchdog(ELOQUA_WATCHDOG_CATEGORY, $message, $variables, WATCHDOG_NOTICE);
    return FALSE;
  }
  $seconds = (int) $seconds;
  variable_set(ELOQUA_VARIABLE_POST_INTERVAL, $seconds);
  return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
//  Webform Setting Database Access Functions


/**
 * Loads a webform settings from the database
 * @hook eloqua_form_load ($webform)
 * @param $nid int
 * @return object
 */
function eloqua_webform_load ($nid) {
  // Get the raw result from the db
  if (!is_numeric($nid)) {
    $type = gettype($nid);
    watchdog(ELOQUA_WATCHDOG_CATEGORY, t('Invalid argument sent to !module_name (!type).',
      array('!module_name' => __FUNCTION__, '!type' => $type)), array(), WATCHDOG_DEBUG);
    return NULL;
  }

  $webform = _eloqua_webform_load($nid);
  if (empty($webform)) {
    return NULL;
  }

  // Send the object around to all of its friends
  foreach (module_implements('eloqua_form_load') as $module_name) {
    $method = $module_name . '_eloqua_form_load';
    $method($webform);
  }

  return $webform;
}


/**
 * Creates a webform settings from the database
 * @hook eloqua_form_create
 * @param $webform object
 * @return boolean
 */
function eloqua_webform_create ($webform) {
  if (!is_object($webform)) {
    $type = gettype($webform);
    watchdog(ELOQUA_WATCHDOG_CATEGORY, t('Invalid argument sent to !module_name (!type).',
      array('!module_name' => __FUNCTION__, '!type' => $type)));
    return FALSE;
  }

  $result = _eloqua_webform_create($webform);
  if (!$result) {
    // if we were unable to create the payment plan.  Can't go on.
    return FALSE;
  }

  foreach (module_implements('eloqua_form_create') as $module_name) {
    $method = $module_name . '_eloqua_form_create';
    $method($webform);
  }

  _eloqua_webform_update($webform);
  return $result;
}


/**
 * Updates a webform settings from the database
 * @hook eloqua_form_update
 * @param $webform object
 * @return boolean
 */
function eloqua_webform_update ($webform) {
  if (!is_object($webform)) {
    $type = gettype($webform);
    watchdog(ELOQUA_WATCHDOG_CATEGORY, t('Invalid argument sent to !module_name (!type).',
      array('!module_name' => __FUNCTION__, '!type' => $type)), array(), WATCHDOG_DEBUG);
    return FALSE;
  }

  // Can't update the obvious invalid ppid of '0'...
  if (empty($webform->nid)) {
    return FALSE;
  }

  foreach (module_implements('eloqua_form_update') as $module_name) {
    $method = $module_name . '_eloqua_form_update';
    $method($webform);
  }

  // Save the result to the database
  $result = _eloqua_webform_update($webform);
  return $result;
}


/**
 * Deletes a webform settings object from the database
 * @param $nid integer
 * @return boolean
 */
function eloqua_webform_delete ($nid) {
 if (!is_numeric($nid)) {
    $type = gettype($nid);
    watchdog(ELOQUA_WATCHDOG_CATEGORY, t('Invalid argument sent to !module_name (!type).',
      array('!module_name' => __FUNCTION__, '!type' => $type)), array(), WATCHDOG_DEBUG);
    return FALSE;
  }

  // Can't update the obvious invalid ppid of '0'...
  if (empty($nid)) {
    return FALSE;
  }

  foreach (module_implements('eloqua_form_delete') as $module_name) {
    $method = $module_name . '_eloqua_form_delete';
    $method($nid);
  }

  $result = _eloqua_webform_delete($nid);
  return $result;
}


/**
 * Loads the webform object from the database
 * @param $nid int
 * @return object
 */
function _eloqua_webform_load ($nid) {
  $result = db_select(ELOQUA_SCHEMA_WEBFORM)
    ->fields(ELOQUA_SCHEMA_WEBFORM)
    ->condition(ELOQUA_WEBFORM_FIELD_NODE_ID, $nid)
    ->execute()
    ->fetchAll(PDO::FETCH_OBJ);
  $result = _eloqua_unserialize_data_column($result);
  $result = array_shift($result);
  return $result;
}



/**
 * Creates the webform object from the database
 * @param $webform
 * @return boolean
 */
function _eloqua_webform_create ($webform) {
  $index = array(
    ELOQUA_WEBFORM_FIELD_NODE_ID => $webform->{ELOQUA_WEBFORM_FIELD_NODE_ID},
  );
  $fields = array(
    ELOQUA_WEBFORM_FIELD_FORM_NAME => $webform->{ELOQUA_WEBFORM_FIELD_FORM_NAME},
    ELOQUA_WEBFORM_FIELD_ACTIVE => $webform->{ELOQUA_WEBFORM_FIELD_ACTIVE},
    ELOQUA_WEBFORM_FIELD_DATA => serialize($webform->{ELOQUA_WEBFORM_FIELD_DATA}),
  );

  $result = _eloqua_db_insert_update($index, $fields);
  return $result;
}


/**
 * Updates the webform object from the database
 * @param $webform object
 * @return boolean
 */
function _eloqua_webform_update ($webform) {
  $fields = array(
    ELOQUA_WEBFORM_FIELD_FORM_NAME => $webform->{ELOQUA_WEBFORM_FIELD_FORM_NAME},
    ELOQUA_WEBFORM_FIELD_ACTIVE => $webform->{ELOQUA_WEBFORM_FIELD_ACTIVE},
    ELOQUA_WEBFORM_FIELD_DATA => serialize($webform->{ELOQUA_WEBFORM_FIELD_DATA}),
  );
  $result = db_update(ELOQUA_SCHEMA_WEBFORM)
    ->fields($fields)
    ->condition(ELOQUA_WEBFORM_FIELD_NODE_ID, $webform->{ELOQUA_WEBFORM_FIELD_NODE_ID})
    ->execute();

  $result = (is_numeric($result)) ? ($result > 0) : FALSE;
  return $result;
}


/**
 * Deletes a webform object from the database
 * @param $nid int
 * @return boolean
 */
function _eloqua_webform_delete ($nid) {
  $result = db_delete(ELOQUA_SCHEMA_WEBFORM)
    ->condition(ELOQUA_WEBFORM_FIELD_NODE_ID, $nid)
    ->execute();
  $result = (is_numeric($result)) ? ($result > 0) : FALSE;
  return $result;
}



/**
 * Returns the defined scripts directory
 * @return int
 */
function eloqua_get_scripts_directory() {
  $value = variable_get(ELOQUA_VARIABLE_SCRIPTS_DIRECTORY, ELOQUA_VARIABLE_SCRIPTS_DIRECTORY_DEFAULT);
  return $value;
}


/**
 * Returns the cron processing batch size
 * @return int
 */
function eloqua_get_batch_size() {
  $batch_size = (int) variable_get(ELOQUA_SETTINGS_BATCH_SIZE, ELOQUA_SETTINGS_BATCH_SIZE_DEFAULT);
  return $batch_size;
}


////////////////////////////////////////////////////////////////////////////////
//  Completed Post Handling


function eloqua_post_get_batch () {
  $result_set = db_select(ELOQUA_SCHEMA_SAVED_POSTS)
    ->fields(ELOQUA_SCHEMA_SAVED_POSTS)
    ->condition(ELOQUA_POST_FIELD_STATUS, ELOQUA_STATUS_NEW)
    ->range(0, eloqua_get_batch_size())
    ->execute()
    ->fetchAll();
  $result = _eloqua_unserialize_data_column($result_set);
  return $result;
}

/**
 * Loads a post form the database
 * @hook eloqua_post_load ($post)
 * @param $post_id int
 * @return object
 */
function eloqua_post_load ($post_id) {
  // Get the raw result from the db
  if (!is_numeric($post_id)) {
    $type = gettype($post_id);
    watchdog(ELOQUA_WATCHDOG_CATEGORY, t('Invalid argument sent to !module_name (!type).',
      array('!module_name' => __FUNCTION__, '!type' => $type)), array(), WATCHDOG_DEBUG);
    return NULL;
  }

  $post = _eloqua_post_load($post_id);
  if (empty($post)) {
    return NULL;
  }

  // Send the object around to all of its friends
  foreach (module_implements('eloqua_post_load') as $module_name) {
    $method = $module_name . '_eloqua_post_load';
    $method($post);
  }

  return $post;
}


/**
 * Creates a post into the database
 * @hook eloqua_post_create
 * @param $post object
 * @return int
 *  Post ID
 */
function eloqua_post_create ($post) {
  if (!is_object($post)) {
    $type = gettype($post);
    watchdog(ELOQUA_WATCHDOG_CATEGORY, t('Invalid argument sent to !module_name (!type).',
      array('!module_name' => __FUNCTION__, '!type' => $type)));
    return FALSE;
  }

  $result = _eloqua_post_create($post);
  if (!$result) {
    // if we were unable to create the payment plan.  Can't go on.
    return FALSE;
  }

  $post->{ELOQUA_POST_FIELD_POST_ID} = $result;

  foreach (module_implements('eloqua_post_create') as $module_name) {
    $method = $module_name . '_eloqua_post_create';
    $method($post);
  }

  _eloqua_post_update($post);
  return $result;
}


/**
 * Updates a post from the database
 * @hook eloqua_post_update
 * @param $post object
 * @return boolean
 */
function eloqua_post_update ($post) {
  if (!is_object($post)) {
    $type = gettype($post);
    watchdog(ELOQUA_WATCHDOG_CATEGORY, t('Invalid argument sent to !module_name (!type).',
      array('!module_name' => __FUNCTION__, '!type' => $type)), array(), WATCHDOG_DEBUG);
    return FALSE;
  }

  // Can't update the obvious invalid ppid of '0'...
  if (empty($post->{ELOQUA_POST_FIELD_POST_ID})) {
    return FALSE;
  }

  foreach (module_implements('eloqua_post_update') as $module_name) {
    $method = $module_name . '_eloqua_post_update';
    $method($post);
  }

  // Save the result to the database
  $result = _eloqua_post_update($post);
  return $result;
}


/**
 * Deletes a post object from the database
 * @param $post_id integer
 * @return boolean
 */
function eloqua_post_delete ($post_id) {
 if (!is_numeric($post_id)) {
    $type = gettype($post_id);
    watchdog(ELOQUA_WATCHDOG_CATEGORY, t('Invalid argument sent to !module_name (!type).',
      array('!module_name' => __FUNCTION__, '!type' => $type)), array(), WATCHDOG_DEBUG);
    return FALSE;
  }

  // Can't update the obvious invalid ppid of '0'...
  if (empty($post_id)) {
    return FALSE;
  }

  foreach (module_implements('eloqua_post_delete') as $module_name) {
    $method = $module_name . '_eloqua_post_delete';
    $method($post_id);
  }

  $result = _eloqua_post_delete($post_id);
  return $result;
}


/**
 * Loads the post object from the database
 * @param $post_id int
 * @return object
 */
function _eloqua_post_load ($post_id) {
  $result_set = db_select(ELOQUA_SCHEMA_SAVED_POSTS)
    ->condition(ELOQUA_POST_FIELD_POST_ID, $post_id)
    ->execute()
    ->fetchAll();
  $result = _eloqua_unserialize_data_column($result_set);
  $result = array_shift($result);
  return $result;
}


/**
 * Creates the post object from the database
 * @param $post
 * @return int
 *  Newly created Post_id
 */
function _eloqua_post_create ($post) {
  $fields = array(
    ELOQUA_POST_FIELD_FORM_ID => $post->{ELOQUA_POST_FIELD_FORM_ID},
    ELOQUA_POST_FIELD_POST_TIME => $post->{ELOQUA_POST_FIELD_POST_TIME},
    ELOQUA_POST_FIELD_STATUS => $post->{ELOQUA_POST_FIELD_STATUS},
    ELOQUA_POST_FIELD_DATA => serialize($post->{ELOQUA_POST_FIELD_DATA}),
  );
  $result = db_insert(ELOQUA_SCHEMA_SAVED_POSTS)
    ->fields($fields)
    ->execute();
  $result = ($result !== FALSE && $result !== NULL);
  return $result;
}


/**
 * Updates the post object from the database
 * @param $post object
 * @return boolean
 */
function _eloqua_post_update ($post) {
  $fields = array(
    ELOQUA_POST_FIELD_FORM_ID => $post->{ELOQUA_POST_FIELD_FORM_ID},
    ELOQUA_POST_FIELD_POST_TIME => $post->{ELOQUA_POST_FIELD_POST_TIME},
    ELOQUA_POST_FIELD_STATUS => $post->{ELOQUA_POST_FIELD_STATUS},
    ELOQUA_POST_FIELD_DATA => serialize($post->{ELOQUA_POST_FIELD_DATA}),
  );
  $result = db_update(ELOQUA_SCHEMA_SAVED_POSTS)
    ->fields($fields)
    ->condition(ELOQUA_POST_FIELD_POST_ID, $post->{ELOQUA_POST_FIELD_POST_ID})
    ->execute();
  $result = (is_numeric($result)) ? ($result > 0) : FALSE;
  return $result;
}


/**
 * Deletes a post object from the database
 * @param $post_id int
 * @return boolean
 */
function _eloqua_post_delete ($post_id) {
  $result = db_delete(ELOQUA_SCHEMA_SAVED_POSTS)
    ->condition(ELOQUA_POST_FIELD_POST_ID, $post_id)
    ->execute();
  $result = (is_numeric($result)) ? ($result > 0) : FALSE;
  return $result;
}


/**
 * Goes through the result set and unserialises any 'data' columns
 * @param  $result_set
 * @return array
 */
function _eloqua_unserialize_data_column($result_set) {
  $result = array();
  if (!is_array($result_set)) {
    $result_set = array();
  }

  foreach ($result_set as $row) {
    $row->data = unserialize($row->data);
    $result[] = $row;
  }

  return $result;
}


/**
 * Query the Database and mimic a INSERT ... ON DUPLICATE KEY UPDATE
 *
 * Because there isn't a "Drupal" way to do a INSERT ... ON DUPLICATE KEY UPDATE using the
 * abstraction layer they provided.
 *
 * Also, when using InnoDB tables, you cannot actually retrieve the last-insert-id reliably
 * So, all in all, this just returns a TRUE/FALSE now, as opposed to the Drupal 6 Version
 * @param  $index
 * @param  $fields
 * @return bool
 */
function _eloqua_db_insert_update($index, $fields) {
  $insert_fields = $index + $fields;
  $do_update = FALSE;
  try {
    $result = db_insert(ELOQUA_SCHEMA_WEBFORM)
      ->fields($insert_fields)
      ->execute()
    ;
    // No longer able to return the last inserted Id.
    if ($result !== FALSE) {
      $result = TRUE;
    }
  } catch (PDOException $e) {
    // Constraint Error
    if ($e->getCode() == '23000') {
      $do_update = TRUE;
    }
  }

  // Attempt Update
  if ($do_update) {
    $query = db_update(ELOQUA_SCHEMA_WEBFORM)
      ->fields($fields);
    foreach ($index as $key => $value) {
      $query->condition($key, $value);
    }

    try {
      $result = $query->execute();
    }
    catch (PDOException $e) {
      return FALSE;
    }

    // Rows Updated
    if (is_numeric($result)) {
      $result = (bool) $result;
    }
    else {
      // Probably failed.
      $result = FALSE;
    }
  }
}

// End of file
